# ingperl.pl
#
# IngPerl - A Ingres Database Interface for Perl 4
#
# Formally sqlperl by Ted Lemon.
# Now developed and maintained by Tim Bunce, timbo@ig.co.uk
# Copyright 1994 Tim Bunce and Ted Lemon 
# The latest version is held in ftp.demon.co.uk:/pub/perl/db/ingperl
#
# IngPerl is supplied under the same terms as perl itself,
# e.g., either GNU GPL or Artistic License at your choice.
#
# IngPerl comes with no warranty - it works for me - it may not
# work for you. If it trashes your database I am not responsible!
#
# This file should be included in all applications using ingperl
# in order to help ensure that scripts will remain compatible with
# new releases of ingperl.

# ---------------------------------------------------------------------

# IngPerl Functions:
#
#
# &sql('...');
#
#	This functions should be used to
#
#	1: connect to a database:
#		&sql("connect database_name [-sqloptions]");
#		where sqloptions are the options defined in the manual
#		for the sql command. For example:
#		&sql("connect database_name -uusername -xw -Rrole -Ggroup -l");
#		Returns true else undef on error.
#
#	2: disconnect from a database:
#		&sql("disconnect");
#		Note that ingperl will complain if a transaction is active.
#		You should &sql_exec 'commit' or 'rollback' before disconnect.
#		Returns true else undef on error (unlikely!).
#		Note that an ingres bug means that $sql_error will contain an
#		error message (E_LQ002E query issued outside of a session) even
#		though the disconnect worked ok.
#
#	3: prepare a statement:
#		&sql("select ...");
#		Returns true else undef on error.
#
#		If a non-select statement is prepared it will be executed at once.
#
#		This function cannot be used to prepare the following statements:
#			call, get dbevent, inquire_sql, execute immediate,
#			execute procedure, execute, prepare to commit, prepare, set.
#		Some of these can be performmed by the &sql_exec() function.
#
#
# &sql_exec('...');
#
#	Execute an sql statement immediately. This function should be used
#	to issue Commit, Rollback, Insert, Delete and Update statements.
#	Returns true else undef on error.
#
#	It is also often used to execute 'set' commands. For example:
#		&sql_exec('set autocommit on');
#		&sql_exec('set session with on_error=rollback transaction');
#		&sql_exec('set lockmode readlock=nolock');
#		&sql_exec('set qep');
#
#	This function cannot be used to prepare the following statements:
#		call, get dbevent, inquire_sql, prepare to commit.
#
#
# @values = &sql_fetch;
#
#	Fetch the next record of data returned from the last prepared
#	select statement. When all records have been returned &sql_fetch
#	will close the select statement cursor and return an empty array.
#	For example:
#		&sql('select * from iitables') || die $sql_error;
#		while(@values = &sql_fetch){
#			...
#		}
#	Null values are returned as undef elements of the array.
#
#
# &sql_close;
#
#	This function needs to be called *only* if you do not use
#	&sql_fetch to fetch *all* the records *and* you wish to close
#	the cursor as soon as possible (to release locks etc).
#	Otherwise ignore it. Always returns true.


# IngPerl Functions to describe the currently prepared statement.
# These functions all return an array with one element for each
# field in the query result.
#
# 
# @types = &sql_types;
#
#	Returns a list of sprintf type letters to indicate the generic
#	type if each field: 'd' (int), 'f' (float), or 's' (string).
#
#
# @ingtypes = &sql_ingtypes;
#
#	Returns a list of specific ingres type numbers:
#	30 - integer
#	31 - float (money type is returned as a float)
#	20 - char  (date type is returned as a char)
#	21 - varchar
#
#
# @lengths = &sql_lengths;
#
#	Returns a list if ingres data type lengths.
#	For strings the length is the maximum width of the field.
#	For numbers it is the number of bytes used to store the
#	binary representation of the value, 1, 2, 4 or 8.
#
#
# @nullable = &sql_nullable;
#
#	Returns a list of boolean values (0 or 1's). A 1 indicates
#	that the corresponding field may return a null value.
#
#
# @names = &sql_names;
#
#   Returns a list of field names.


# IngPerl Variables:
#
#
# $sql_version (read only)
#
#	A constant string compiled into ingperl containing the major
#	and minor version numbers of ingperl, the patchlevel and the
#	date that the ingperl binary was built. For example:
#		ingperl 2.0 pl0 (built Apr  8 1994 13:17:03)
#
# $sql_error (read only)
#
#	Contains the error message text of the current ingres error.
#	Is empty if last statement succedded. For example:
#		print "$sql_error\n" if $sql_error;
#
# $sql_sqlcode (read only)
#
#	The current value of sqlda.sqlcode. Only of interest in more
#	sophisticated applications. Typically 0, <0 on error,
#	100=no more rows, 700=message, 710=dbevent.
#
# $sql_rowcount (read only)
#
#	After a successful Insert, Delete, Update, Select, Modify,
#	Create Index, Create Table As Select or Copy this variable
#	holds the number of rows affected.
#
# $sql_readonly (default 1)
#
#	If true then prepared sql statements are given read only cursors
#	this is generally a considerable performance gain.
#
# $sql_showerrors (default 0)
#
#	If true then ingres error and warning messages are printed by
#	ingperl as they happen. Very useful for testing.
#
# $sql_debug (default 0)
#
#	If ingperl has been compiled with -DINGPERL_DEBUG then setting this
#	variable true will enable debugging messages from ingperl internals.
#
#


# IngPerl Library Functions
#
#
# @row1 = &sql_eval_row1('select ...');
#
#	Execute a select statement and return the first row.
#
#
# @col1 = &sql_eval_col1('select ...');
#
#	Execute a select statement and return the first column.
#

# ---------------------------------------------------------------------


# Just incase this perl is not an ingperl don't crash so badly:
if (! defined &sql ){
	eval <<END;
	sub sql{ return undef; }
	sub sql_exec{ return undef; }
	$sql_error="This perl does not include ingperl extensions";
	$sql_sqlcode = -1;
END
}


# Library function to execute a select and return first row
sub sql_eval_row1{
	&sql(@_);
	return undef if $sql_error;
	local(@row) = &sql_fetch;	# fetch one row
	&sql_close;					# close the cursor
	@row;
}

# Library function to execute a select and return first col
sub sql_eval_col1{
	&sql(@_);
	return undef if $sql_error;
	local(@row, @col);
	while (@row = &sql_fetch){
		push(@col, $row[0]);
	}
	@col;
}


1; # end ingperl.pl
